--[[
	Effects script for CS2D
	by Avobolt (USGN#88318)
]]--

--// SETUP

eff = {}
eff.Table = {}
eff.UpdateMode = "ms100"
eff.Limit = 200
eff.RadiusLimit = 200
eff.AmountLimit = 100

--// ADDING / REMOVING EFFECTS

eff.Create = function(typ, ...)
	-- typ:string x:number y:number time:number amount:number radius:number r:number/string g:number/string b:number/string
	local __error,  __e = false, {}
	local A = {...}
	if (not (#eff.Table < eff.Limit)) then
		print("eff.Create: Effects limit has been reached!")
		__error = true
	end
	if (typ == "fire" or typ == "smoke" or typ == "flare" or typ == "colorsmoke") then
		if A[4] > eff.AmountLimit then A[4] = eff.AmountLimit end
		if A[5] > eff.RadiusLimit then A[5] = eff.RadiusLimit end
		__e = {Type = typ, X = A[1], Y = A[2], Time = tonumber(A[3]), Amount = A[4], Radius = A[5], R = (A[6] or 0), G = (A[7] or 0), B = (A[8] or 0)}
		if (typ == "fire" or typ == "smoke") then __e.R = 0 __e.G = 0  __e.B = 0 end
		table.insert(eff.Table, __e)
	else
		print("eff.Create: Effect name isn't known!")
		__error = true
	end
	if not __error then
		return #eff.Table
	else
		return 0
	end
end
eff.Remove = function(id)
	if id == 0 then eff.Table = {} return end
	eff.Table[id] = nil
end

--// SET FUNCTIONS

eff.SetColor = function(id, r, g, b)
	if (not eff.Table[id]) then print("eff.SetColor: Effect doesn't exist!") return end
	eff.Table[id].R = tonumber(r) or 0
	eff.Table[id].G = tonumber(g) or 0
	eff.Table[id].B = tonumber(b) or 0
end
eff.SetAm = function(id, am)
	if (not eff.Table[id]) then print("eff.SetAm: Effect doesn't exist!") return end
	am = tonumber(am) or 0
	if am > eff.AmountLimit then am = eff.AmountLimit end
	eff.Table[id].Amount = am
end
eff.SetRad = function(id, radius)
	if (not eff.Table[id]) then print("eff.SetRad: Effect doesn't exist!") return end
	radius = tonumber(radius) or 0
	if radius > eff.RadiusLimit then radius = eff.RadiusLimit end
	eff.Table[id].Radius = radius
end
eff.SetPos = function(id, x, y)
	if (not eff.Table[id]) then print("eff.SetPos: Effect doesn't exist!") return end
	eff.Table[id].X = tonumber(x) or 0
	eff.Table[id].Y = tonumber(y) or 0
end
eff.SetTime = function(id, time)
	if (not eff.Table[id]) then print("eff.SetTime: Effect doesn't exist!") return end
	eff.Table[id].Time = tonumber(time) or 0
end

--// GET FUNCTIONS

eff.GetColor = function(id)
	if (not eff.Table[id]) then print("eff.GetColor: Effect doesn't exist!") return end
	local e = eff.Table[id]
	return e.R, e.G, e.B
end
eff.GetAm = function(id)
	if (not eff.Table[id]) then print("eff.GetAm: Effect doesn't exist!") return end
	local e = eff.Table[id]
	return e.Amount
end
eff.GetRad = function(id)
	if (not eff.Table[id]) then print("eff.GetRad: Effect doesn't exist!") return end
	local e = eff.Table[id]
	return e.Radius
end
eff.GetPos = function(id)
	if (not eff.Table[id]) then print("eff.GetPos: Effect doesn't exist!") return end
	local e = eff.Table[id]
	return e.X, e.Y
end
eff.GetTime = function(id)
	if (not eff.Table[id]) then print("eff.GetTime: Effect doesn't exist!") return end
	local e = eff.Table[id]
	return e.Time
end

--// LUA HOOK TO UPDATE EFFECTS

addhook(eff.UpdateMode, "eff.Update")
eff.Update = function()
	local _time = 1000
	if eff.UpdateMode == "ms100" then
		_time = 100
	elseif eff.UpdateMode == "always" then
		_time = 50
	end
	for id, t in pairs(eff.Table) do
		parse('effect "'..t.Type..'" '..t.X..' '..t.Y..' '..t.Amount..' '..t.Radius..' '..t.R..' '..t.G..' '..t.B)
		if (eff.GetTime(id) >= 0) then
			eff.SetTime(id, t.Time - _time)
			if (eff.GetTime(id) <= 0) then
				eff.Remove(id)
			end
		end
	end	
end

addhook("startround", "eff.RemoveAll")
eff.RemoveAll = function()
	eff.Remove(0)
end